#####################
### Preliminaries ###
#####################
require(spdep)
require(shapefiles)
require(CARBayes)
require(INLA)
require(rgdal)
require(rgeos)

############
### Data ###
############
# Reading in the observed counts
observed <- read.csv(file="copdmortalityobserved.csv", row.names=1)

# Reading in the expected counts
expected <- read.csv(file="copdmortalityexpected.csv", row.names=1)

# Viewing the first 6 rows of the observed counts
head(observed)

# Viewing the first 6 rows of the expected counts
head(expected)

# Summarising the observed counts
summary(observed)

# Summarising the expected counts
summary(expected)

# Read in shapefiles for England split by local authorities
shp <- read.shp(shp.name="englandlocalauthority.shp")
shp$shp[[23]]$points <- shp$shp[[23]]$points[-c(460, 461, 462, 463, 464, 465), ]

# Read in information for England split by local authorities
dbf <- read.dbf(dbf.name="englandlocalauthority.dbf")
dbf$dbf <- dbf$dbf[ ,c(2,1,3:7)]

#########################
### Creating raw SMRs ###
#########################
# Creating raw SMRs
SMR_raw <- observed[ ,-1]/expected

# Changing column names
names(SMR_raw) <- c("SMR2001_raw", "SMR2002_raw", "SMR2003_raw", "SMR2004_raw",
                    "SMR2005_raw", "SMR2006_raw", "SMR2007_raw", "SMR2008_raw", 
                    "SMR2009_raw", "SMR2010_raw")

# Viewing the first 6 rows of SMR_raw
head(SMR_raw)

# Summarising SMR_raw
summary(SMR_raw)

# Attaching the raw SMRs to the the shapefiles
SMRspatial_raw <- combine.data.shapefile(data = SMR_raw, # Dataset to attach
                                         shp  = shp,     # Shapefile
                                         dbf  = dbf)     # Database file

# Breaks for legend of plot
range <- seq(min(SMR_raw$SMR2010_raw)-0.01, max(SMR_raw$SMR2010_raw)+0.01, length.out=11)

# Creating a map of 2010 smoothed estimates
spplot(obj  = SMRspatial_raw,    # Spatial object to be plotted
       zcol = c("SMR2010_raw"),  # Choice of the column  the object you are plotting. 
       xlab = "Easting",     # Label of the x column 
       ylab = "Northing",    # Label of the y column 
       at   = range,         # Break points for colours
       col  = "transparent", # Choice of colour
       col.regions = hsv(0.6, seq(0.2, 1, length.out=10), 1)) # Create a set of colours

#########################################
### Creating Smoothed SMRs using MCMC ###
#########################################
# Creates the neighbourhood
W.nb <- poly2nb(SMRspatial_raw, row.names = rownames(SMRspatial_raw)) 

# Creates a matrix for following function call
W.mat <- nb2mat(W.nb, style="B") 

# Model Formula
formula <- observed$Y2010 ~ 1 + offset(log(expected$E2010))

# Calculating smoothed SMRs using MCMC
model <- S.CARleroux(formula = formula,  # Model Formula
                     family = "poisson", # Choosing Poisson Regression
                     W = W.mat,          # Neighbourhood matrix 
                     burnin = 20000,     # Number of burn in samples
                     n.sample = 100000,  # Number of MCMC samples
                     thin = 10, 
                     fix.rho = TRUE, 
                     rho = 1)

# Calculating the 2010 smoothed SMRs
SMR2010_MCMC <- model$fitted.values / expected$E2010 

# Putting 2010 smoothed SMRs into a dataframe
SMR_smooth_MCMC <- as.data.frame(SMR2010_MCMC, row.names = rownames(observed))

# Looking at the first 6 rows of SMR_smooth_MCMC
head(SMR_smooth_MCMC)

# Summarising SMR_smooth_MCMC
summary(SMR_smooth_MCMC)

# Attaching the Smoothed SMRs extimated from MCMC to a shapefile
SMRspat_smooth_MCMC <- combine.data.shapefile(data = SMR_smooth_MCMC, # Dataset to attach
                                              shp  = shp,        # Shapefile
                                              dbf  = dbf)        # Database file

# Creating a map of 2010 smoothed SMRs extimated from MCMC
spplot(obj  = SMRspat_smooth_MCMC, # Spatial object to be plotted
       zcol = c("SMR2010_MCMC"),   # Choice of the column  the object you are plotting. 
       xlab = "Easting",           # Label of the x column 
       ylab = "Northing",          # Label of the y column 
       at   = range,               # Break points for colours
       col  = "transparent",       # Choice of colour
       col.regions = hsv(0.6, seq(0.2, 1, length.out=10), 1)) # Create a set of colours



###########################################
### Creating Smoothed SMRs using R-INLA ###
###########################################
# Create an INLA format neighbourhood structure
nb2INLA("UK.adj",W.nb)

# Creating an ID to match observations ot area
ID <- 1:nrow(observed)

# Extracting observed values
obs <- observed$Y2010 

# Extracting expected values
exp <- expected$E2010 

# Creating a dataset to input to INLA
dat <- data.frame(obs, exp, ID)

# Formula for Poisson log-normal model with spatial effects
formula <- obs ~ 1 + f(ID, model="besag", graph="UK.adj")

# Fitting the model with R-INLA
mod <- inla(formula, 
            family = "poisson", 
            data = dat,
            E = exp, # Expected Values
            control.predictor=list(compute=TRUE)) # Create predictions

# Extracting the 2010 smoothed SMRs
SMR2010_INLA <- mod$summary.fitted.values$mean

# Putting 2010 smoothed SMRs into a dataframe
SMR_smooth_INLA <- as.data.frame(SMR2010_INLA, 
                                 row.names = rownames(observed))

# Looking at the first 6 rows of SMR_smooth_INLA
head(SMR_smooth_INLA)

# Summarising SMR_smooth_INLA
summary(SMR_smooth_INLA)

# Attaching the Smoothed SMRs extimated from INLA to a shapefile
SMRspat_smooth_INLA <- combine.data.shapefile(data = SMR_smooth_INLA, # Dataset to attach
                                              shp  = shp,        # Shapefile
                                              dbf  = dbf)        # Database file

# Creating a map of 2010 smoothed SMRs extimated from R-INLA
spplot(obj  = SMRspat_smooth_INLA,  # Spatial object to be plotted
       zcol = c("SMR2010_INLA"),    # Choice of the column  the object you are plotting. 
       xlab = "Easting",            # Label of the x column 
       ylab = "Northing",           # Label of the y column 
       at   = range,                # Break points for colours
       col  = "transparent",        # Choice of colour
       col.regions = hsv(0.6, seq(0.2, 1, length.out=10), 1)) # Create a set of colours

#########################################################
### Comparison of MCMC and R-INLA for smoothing risks ###
#########################################################
# Creating a dataframe with the estimates of MCMC and INLA to analyse
smoothed_2010 <- data.frame(SMR2010_MCMC,
                            SMR2010_INLA, 
                            row.names = rownames(observed))

# Calculating a summary of the differences between MCMC and INLA
summary(smoothed_2010$SMR2010_MCMC - smoothed_2010$SMR2010_INLA)

# Attaching the Smoothed SMRs extimated from INLA to a shapefile
SMRspat_smooth <- combine.data.shapefile(data = smoothed_2010, # Dataset to attach
                                              shp  = shp,        # Shapefile
                                              dbf  = dbf)        # Database file

# Creating a map of 2010 smoothed SMRs extimated from R-INLA
spplot(obj  = SMRspat_smooth,  # Spatial object to be plotted
       xlab = "Easting",            # Label of the x column 
       ylab = "Northing",           # Label of the y column 
       at   = range,                # Break points for colours
       col  = "transparent",        # Choice of colour
       col.regions = hsv(0.6, seq(0.2, 1, length.out=10), 1)) # Create a set of colours



